/*       -*- C++ -*- */
/** \file
 ********************************************************************
 * $URL: svn+ssh://nalw02/home/alg/schlemme/svn/cextools/trunk/include/cexfile/context.h $
 * $Id: context.h 12 2008-02-26 15:45:35Z schlemme $
 * $Date: 2008-02-26 16:45:35 +0100 (Di, 26 Feb 2008) $
 * $Revision: 12 $
 * \author Tobias Schlemmer <keinstein@users.berlios.de>
 * $Author: schlemme $ 
 * \license LGPL
 * 
 * Context class.
 ********************************************************************
 */	  
#ifndef CEXFILE_CONTEXT_H
#define CEXFILE_CONTEXT_H

#include <iostream>
#include <vector>
#include "cexfile/attribute.h"
#include "cexfile/object.h"

namespace cexfile {
  /// a formal context
  class context {
    /// identifier to denote the context
    Glib::ustring id;

    /// XML Node describing the context
    const xmlpp::Node * node;
    
    /// set of attributes
    attribute_s attributes;

    /// set of objects;
    object_s objects;

    /// type of context
    enum { 
      noContext, ///< not yet initialized
      binaryContext ///< binary context
    } type;

  public:
    /// default constructor
    context():id(""),
	      node(NULL),
	      attributes(),
	      objects(),
	      type(noContext) {}

    /// standard constructor
    /** \param Node XML node to be parsed
     */
    context(const xmlpp::Node * Node);
    
    /// returns the identifier of the context
    /** \return String representing the identifier */
    Glib::ustring & get_id() { return id; }

    /// return the set of objects
    /** \return reference to the set of objects */
    const object_s & get_objects() { return objects; }
    
    /// return the attribute set
    /** \return reference to the set of attributes */
    const attribute_s & get_attributes() { return attributes; }
    
    /// return the number of objects
    /** \return number of objcects in the object set
     */
    int get_object_count() const { return objects.size(); }

    /// Do we have the attribute?
    /** \param i intent to be compared
	\todo Implement or remove it 
    */
    bool has_attribute (intent & i) {
      
    }
  };

  /// a set mapping context identifiers to contexts  
  typedef std::map<Glib::ustring,context> context_s;

  /// add a context to the context set
  /** \relates context_s
      \relates context
      \param s set to add the concept
      \param c context to be added 
      \return reference to the set \p s
  */
  inline context_s& operator+=(context_s& s, context c) {
    s[c.get_id()] = c;
  }

  inline context::context(const xmlpp::Node * Node):node(Node),
						    attributes(),
						    objects()
  {
    if (!Node) throw xmlpp::validity_error("Empty parent Node for contexts.");

#ifdef DEBUG
    std::cerr << "Context: " << Node -> get_path() << std::endl;
#endif
    
    const xmlpp::Element* nodeElement = 
      dynamic_cast<const xmlpp::Element*>(Node);
    
    if (nodeElement) {
      xmlpp::Attribute * Attr = nodeElement->get_attribute("Identifier");
      if (Attr) 
	id = Attr -> get_value();
      else throw xmlpp::validity_error("Context has no Identifier");

      Attr = nodeElement->get_attribute("Type");
      if (Attr) {
	Glib::ustring s = Attr->get_value();
	if (s=="Binary") type = binaryContext;
	else throw xmlpp::validity_error("Unsupported context type.");
      } else throw xmlpp::validity_error("No Context type given.");
    } else 
      throw xmlpp::validity_error
	("Context must have an id and an Type attribute.");

    xmlpp::NodeSet att = Node->find("./Attributes/Attribute");

    for (xmlpp::NodeSet::iterator i = att.begin(); i != att.end(); i++){
      attributes += attribute(*i);
    }

    att = Node->find("./Objects/Object");
    
    int id = 0;
    for (xmlpp::NodeSet::iterator i = att.begin(); i != att.end(); i++){
      objects += object(*i,id++);
    }
     
  }
  
}

#endif /* CEXFILE_CONTEXT_H */
