/*       -*- C++ -*- */
/** \file
 ********************************************************************
 * $URL: svn+ssh://nalw02/home/alg/schlemme/svn/cextools/trunk/include/cexfile/object.h $
 * $Id: object.h 12 2008-02-26 15:45:35Z schlemme $
 * $Date: 2008-02-26 16:45:35 +0100 (Di, 26 Feb 2008) $
 * $Revision: 12 $
 * \author Tobias Schlemmer <keinstein@users.berlios.de>
 * $Author: schlemme $ 
 * \license LGPL
 * 
 * Object class.
 ********************************************************************
 */	  
#ifndef CEXFILE_OBJECT_H
#define CEXFILE_OBJECT_H

#include "cexfile/utility.h"
#include "cexfile/intent.h"


namespace cexfile {
  /// an object of the formal context
  class object:public object_base {
  public:
    /// type of the identifier
    typedef int idtype;

    /// not initialized means noId as identifier
    static const idtype noId = -1;

  protected:
    /// name of the object
    Glib::ustring name;

    /// identifier of the object
    idtype id;

  public:
    /// default constructor
    object():object_base(),name(""),id(noId) {}

    /// standard constructor
    /** \param Node root XML node of the object description
	\param i identifier to assign this object to
    */
    object(const xmlpp::Node * Node, int i=noId);
    
    /// compare this object to another in linear order
    /** \param o other object 
	\retval true if we are lower in linear order
	\retval false otherwise
    */
    bool operator < (const object & o) const 
    {
      return (object_base)*this < (object_base) o;
    }

    /// Convertion to a string.
    /** \return Name of the object 
     */
    operator Glib::ustring () const {
      return name;
    }
  };

  /// set of objects (vector)
  typedef std::vector<object> object_v;
  
  /// a multiset of objects (ordered)
  typedef std::multiset<object> object_s;

  /// a set of pointers referencing objects
  typedef std::set<const object *> object_rs;


  //  typedef std::vector<object> object_s;

  /// add an object to a multiset
  /** \relates object_s
      \relates object
      \param s multiset to be expanded
      \param o object to be added
      \return reference to the multiset 
  */
  inline object_s& operator += (object_s& s, object o) 
  {
    s.insert(o);
    //    s.push_back(o);
    return s;
  }

  /// append an object to a set
  /** \relates object_v
      \relates object
      \param v vector to be expanded
      \param o object to be added
      \return reference to the vector 
  */
  inline object_v& operator += (object_v& v, object o) {
    v.push_back(o);
    return v;
  }

  /// add an object reference to a object pointer set
  /** \relates object_rs
      \relates object
      \param s set to be expanded
      \param o object, whose address shall be added
      \return reference to the set 
  */
  inline object_rs& operator += (object_rs& s, const object &o) {
    s.insert(&o);
  }

  /// expand a reference set by the other
  /** \relates object
      \param s1 set to be expanded
      \param s2 set to be added
      \return reference to the first set containing the union of both 
  */
  inline object_rs & operator += (object_rs & s1, const object_rs & s2) {
    object_rs::iterator i1 = s1.begin();
    for (object_rs::const_iterator i2 = s2.begin() ; i2 != s2.end() ; i2++)
      i1=s1.insert(i1,*i2);
  }

  /// union of two reference sets
  /** \relates object
      \param s1 first set
      \param s2 second set
      \return a new %object containing the union of \p s1 and \p s2
  */
  inline object_rs operator + (const object_rs& s1, const object_rs &s2) {
    object_rs ret = s1;
    return (ret += s2);
  }

  inline object::object(const xmlpp::Node * Node,int i):object_base(Node),
							id(i)
  {
    if (!Node) 
      throw xmlpp::validity_error("Empty parent Node for objects.");

#ifdef DEBUG
    std::cerr << "Object: " << Node -> get_path() << std::endl;
#endif

    name = parse_name(Node);
  }

  /*
  inline bool operator < (const object & o) const 
  {
    if (*this < o) return true;
    else if (attributes != o.attributes) return false;
    return id < o.id;
  }
  */

}


#endif /* CEXFILE_OBJECT_H */
